#include <joyos.h>
#include <math.h>

//Values used in function
//Both values currently in inches
#define ROBOT_HEIGHT (10)
#define CAMERA_HEIGHT (129)
#define VPS_TO_FEET (443.4)
#define THETA_UNITS_TO_DEGREES (0.08793)

//New Structures

typedef struct RobotPosition
    {
        int x;
        int y;
        int theta;
    }RobotPosition;

//Functions from other files

	RobotPosition getCurrentPosition();
    int getSimpleHeading(int, int, int, int);
    float getSimpleDistanceTo(int, int, int, int);
    int angleModulus(int);
    void proportionalDrive(int, int, int);
    void turn(int);
    void stop();
	void goToRoller();
	void goTowards(int, int, int);
	int currentTerritory(int, int);

//Functions in this file
int nearestToMiasmador(bool, int, int);
int findClosestMine(bool); //find closest mine -- checks current position -> checks smallest rate limit -> checks closest territory -> checks if owns
RobotPosition findOpponent(); //rough estimate of opponent's location
//void avoid(); //goes around opponent if gets too close
void backOut(); //time out for head-on collision; backs up and turns away
int findClosestNotOurs();

int nearestToMiasmador(bool teamblue, int terr1, int terr2) //terr2 > terr1
{
	int nearest;

	if(teamblue)
	{
		if((terr1 == 1 || terr1 == 2 || terr1 == 3) && terr2 == 0)
			nearest = terr1;
		else
			nearest = terr2;

	}
	else //if team red
	{
		if((terr1 == 1 || terr1 == 2 || terr1 == 3) && terr2 == 0)
			nearest = terr2;
		else if(terr1 == 2 && terr2 == 1)
			nearest = terr2;
		else
			nearest = terr1;
	}

	return nearest;
}

int findClosestMine(bool teamBlue)
{
	copy_objects();
	int lowestRate = 60; //highest rate_limit could be
	int lowestTerritory = 7; //gives lowest rate_limit territory that's closest

	//Check current position's availability
	RobotPosition me = getCurrentPosition();
	int myTerr = currentTerritory(me.x, me.y);

    RobotPosition you = findOpponent();
    int yourTerr = currentTerritory(you.x, you.y);

	if((game.territories[myTerr].rate_limit < 5) && (yourTerr != myTerr))
	{
		lowestTerritory = myTerr;
        lowestRate = game.territories[myTerr].rate_limit;
	}
	else	//Find other territory
	{
		for(int i=0; i<6; i++)
		{
			int rateLimit = game.territories[i].rate_limit;

			if(rateLimit < lowestRate)
			{
				lowestTerritory = i;
				lowestRate = game.territories[i].rate_limit;
			}
			else if(rateLimit == lowestRate)
			{
				//Find closest territory
				int diff1 = abs(i - myTerr);
				int diff2 = abs(lowestTerritory - myTerr);

				//Max distance = 3; wrap around if diff1 or diff2 = 4 or 5
				if(diff1 == 4)
					diff1 = 2;
				if(diff2 == 4)
					diff2 = 2;

				if(diff1 == 5)
					diff1 = 1;
				if(diff2 == 5)
					diff2 = 1;

				if(diff1 < diff2) //only return i if smaller distance than current lowestTerritory
				{
					lowestTerritory = i;
					lowestRate = game.territories[i].rate_limit;
				}
				else if(diff1 == diff2) //if equal distance, return i only if we own i but not current lowestTerritory
				{
					if(game.territories[i].owner == 14 && game.territories[lowestTerritory].owner != 14)
					{
						lowestTerritory = i;
						lowestRate = game.territories[i].rate_limit;
					}

					//else return whichever does not hold an opponent
					else if((game.territories[i].owner == 14 && game.territories[lowestTerritory].owner == 14) ||
					(game.territories[i].owner != 14 && game.territories[lowestTerritory].owner !=14))
					{
						//Find opponent's location
						you = findOpponent();
                        yourTerr = currentTerritory(you.x, you.y);

						if(yourTerr != i) //only return i if opponent not in territory
						{
							lowestTerritory = i;
							lowestRate = game.territories[i].rate_limit;
							if (yourTerr != lowestTerritory) //return whichever closest to Miasmador
                            {
                                lowestTerritory = nearestToMiasmador(teamBlue, i, lowestTerritory);
                                lowestRate = game.territories[lowestTerritory].rate_limit;
                            }
                        } //Endif: Opponent not in i
					}// Endif: Both/neither i and lowestTerritory owned by us
				}// Endif: i and lowestTerritory the same distance away
			}//Endif: i will has balls the soonest
		}//Endloop: Testing all possible territories
	}//End-else: Testing territories that we are not in now

    if (lowestRate > 15) //if all territories have rate limits greater than 15 sec, just roll closest not ours. - rest of code in umain.c
    {
        return 7;
    }

	return lowestTerritory;
}//End function

RobotPosition findOpponent()
{
	RobotPosition you;
	copy_objects();
	you.x = game.coords[1].x - game.coords[1].x*ROBOT_HEIGHT/CAMERA_HEIGHT;
	you.y = game.coords[1].y - game.coords[1].y*ROBOT_HEIGHT/CAMERA_HEIGHT;
	you.theta = game.coords[1].theta;
	return you;
}

/*void avoid()
{
	RobotPosition me = getCurrentPosition();
	RobotPosition you = findOpponent();

	while(getSimpleDistanceTo(me.x, me.y, you.x, you.y < VPS_TO_FEET * 7/6) //2 inch margin-of-error
	{
		int direction = getSimpleHeading(me.x, me.y, you.x, you.y);

		if(direction >= 90 && direction <= -90) //opponent is to the left of you OR head/tail-on
		{
			goTowards(100, (me.theta - 30*THETA_UNITS_TO_DEGREES), 0); //turn clockwise and move around opponent
		}
		else if(direction < 90 && direction > -90) //opponent is to the right of you
		{
			goTowards(100, (me.theta + 30*THETA_UNITS_TO_DEGREES), 0); //turn counterclockwise and move around opponent
		}
	}
} */

/*void backOut()
{
	RobotPosition me = getCurrentPosition();
	int startTheta = me.theta;
	uint32_t startTime = get_time();

	//backs up at 45 degree angle for 2 seconds
	while(get_time() - startTime <= 2000)
	{
		goTowards(100, angleModulus(me.theta - 135*THETA_UNITS_TO_DEGREES), -1);
	}
	//readjust theta to old theta
	turn(startTheta);
}*/

int findClosestNotOurs()
{
	copy_objects();

	int closestToClaim = 7; //gives closest territory that's not ours

	//Check current position's availability
	RobotPosition me = getCurrentPosition();
	int myTerr = currentTerritory(me.x, me.y);

	if(game.territories[myTerr].owner != 14)
	{
		closestToClaim = myTerr;
	}
	else	//Find closest other territory
	{
        int smallestDiff = 6;

		for(int i=0; i<6; i++)
		{
			int currentDiff = abs(i - myTerr);

            //Max distance = 3; wrap around if diff1 or diff2 = 4 or 5
            if(currentDiff == 4)
                currentDiff = 2;
            if(currentDiff == 5)
                currentDiff = 1;

			if(game.territories[i].owner != 14 && smallestDiff > currentDiff)
			{
				closestToClaim = i;
				smallestDiff = currentDiff;
			}
			else if(game.territories[i].owner !=14 && smallestDiff == currentDiff) //check if adjacent ones also not ours
			{
				if(i == 0)
				{
					if(game.territories[i+1].owner != 14 || game.territories[5].owner != 14)
					{
						closestToClaim = i;
					}
				}
				else if(i == 5)
				{
					if(game.territories[i-1].owner != 14 || game.territories[0].owner != 14)
					{
						closestToClaim = i;
					}
				}
				else
				{
					if(game.territories[i+1].owner != 14 || game.territories[i-1].owner !=14)
					{
						closestToClaim = i;
					}
				}
				smallestDiff = currentDiff;
			}
		}
    }
    return closestToClaim;
}
